BN: afs
SF: afs.c aft.c attribute.c
SN: list of afs commands
TM: mood lyr img pl
---
N: add
P: AFS_READ | AFS_WRITE
D: Add new audio files to the database.
U: add [-l] [-f] [-v] path...
H: Each given path may be either the full path to an audio
H: file, or the full path of a directory. In case of a
H: directory, all audio files in that directory are added
H: recursively. Only absolute paths are accepted.
H:
H: Options:
H:
H: -a	Add all files. The default is to add only files ending in a
H:	known suffix for a supported audio format.
H:
H: -l	Add files lazily. If the path already exists in the
H:	database, skip this file.  This operation is really cheap.
H:	Use it when adding large directories if only a few files
H:	where added.
H:
H: -f	Force adding/updating. Recompute the audio format handler data
H: 	even if a file with the same path and the same hash value exists.
H:
H: -v	Verbose mode. Print what is being done.
---
N: init
P: AFS_READ | AFS_WRITE
D: Initialize the osl tables for the audio file selector.
U: init [table_name ...]
H: When invoked without arguments, this command creates all tables. Otherwise
H: only the tables given by table_name... are created.
---
N: ls
P: AFS_READ
D: List audio files.
U: ls [-l[s|l|v|m]] [-p] [-a] [-r] [-d] [-s{p|s|l|n|f|c|i|y|b|d|a}] [pattern...]
H: Print a list of all audio files matching pattern.
H:
H: Options:
H:
H: -l	Change listing mode. Defaults to short listing if not given.
H:
H:    -ls: short listing mode
H:    -ll: long listing mode (equivalent to -l)
H:    -lv: verbose listing mode
H:    -lp: parser-friendly mode
H:    -lm: mbox listing mode
H:    -lc: chunk-table listing mode
H:
H: -p	List full path of audio file. If not specified, only the basename
H:	of each file is printed.
H:
H: -a	List only files that are admissible with respect to the current mood or
H:	playlist.
H:
H: -r	Reverse sort order.
H:
H: -d	Print dates as seconds after the epoch.
H:
H: -s	Change sort order. Defaults to alphabetical path sort if not given.
H:
H:    -sp: by path
H:    -sl: by last played time
H:    -ss: by score (implies -a)
H:    -sn: by num played count
H:    -sf: by frequency
H:    -sc: by number of channels
H:    -si: by image id
H:    -sy: by lyrics id
H:    -sb: by bit rate
H:    -sd: by duration
H:    -sa: by audio format
---
N: lsatt
P: AFS_READ
D: List attributes.
U: lsatt [-i] [-l] [-r] [pattern]
H: Print the list of all defined attributes which match the
H: given pattern. If no pattern is given, the full list is
H: printed.
H:
H: Options:
H:
H: -i	Sort attributes by id. The default is to sort alphabetically by name.
H:
H: -l	Print a long listing containing both identifier and attribute name. The
H: 	default is to print only the name.
H:
H: -r	Reverse sort order.
---
N: setatt
P: AFS_READ | AFS_WRITE
D: Set attribute(s) for all files matching a pattern.
U: setatt attribute{+|-}... pattern
H: Set ('+') or unset ('-') the given attributes for all audio files matching
H: pattern.  Example:
H:
H:         setatt rock+ punk+ classic- '*foo.mp3'
H:
H: sets the 'rock' and the 'punk' attribute but unsets the 'classic'
H: attribute of all files ending with 'foo.mp3'.
---
N: addatt
P: AFS_READ | AFS_WRITE
D: Add new attribute(s).
U: addatt attribute1...
H: This adds new attributes to the attribute table. At most 64
H: attributes may be defined.
---
N: mvatt
P: AFS_READ | AFS_WRITE
D: Rename an attribute.
U: mvatt old new
H: Rename attribute old to new.
---
N: check
P: AFS_READ
D: Run integrity checks against osl tables.
U: check [-a] [-m] [-p]
H: Check the audio file table, the mood definitions and all
H: defined playlists and report any inconsistencies found.
H:
H: Options:
H:
H: -a 	Run audio file table checks. Checks for entries in the audio file
H: 	table which are not present in the file system. Moreover, it checks
H: 	whether the lyrics id and all entries in the audio file table are
H: 	valid.
H:
H: -m	Run syntax checks on all defined moods in the mood table.
H:
H: -p	Check all playlists for lines that correspond to files not contained
H: 	in the audio file table.
H:
H: If called without arguments, all three checks are run.
---
N: rmatt
P: AFS_READ | AFS_WRITE
D: Remove attribute(s).
U: rmatt pattern...
H: Remove all attributes matching any given pattern. All information
H: about this attribute in the audio file table is lost.
---
N: rm
P: AFS_READ | AFS_WRITE
D: Remove entries from the audio file table.
U: rm [-v] [-f] [-p] pattern...
H: Delete all entries in the audio file table that match any given pattern.
H: Note that affects the table entries only; paraslash won't touch your
H: audio files in any way.
H: Options:
H:
H: -v	Verbose mode. Explain what is being done.
H:
H: -f	Force mode. Ignore nonexistent files. Don't complain if nothing
H:	was removed.
H:
H: -p	Pathname match. Match a slash in the path only with a slash
H: 	in pattern and not by an asterisk (*) or a question mark
H: 	(?) metacharacter, nor by a bracket expression ([]) containing
H: 	a slash (see fnmatch(3)).
---
N: touch
P: AFS_READ | AFS_WRITE
D: Manipulate the afs data for all audio files matching a pattern.
U: touch [-n=numplayed] [-l=lastplayed] [-y=lyrics_id] [-i=image_id] [-a=amp] [-v] [-p] pattern
H: If no option is given, lastplayed is set to the current time
H: and numplayed is increased by one. Otherwise, only the given
H: options are taken into account.
H:
H: Options:
H:
H: -n	Set numplayed count. The number of times afs has selected this
H: 	audio file for streaming.
H:
H: -l	Set lastplayed time. The last time this audio file was selected.
H: 	Must be given as the number of seconds since the epoch. Example:
H:
H: 		touch -l=$(date +%s) file
H:
H: 	sets the lastplayed time of 'file' to the current time.
H:
H: -y	Set the lyrics id. Specify the lyrics data file associated with
H: 	this audio file.
H:
H: -i	Set the image id. Same as -y, but sets the image.
H:
H: -a	Set the amplification value (0-255). This determines a scaling
H:	factor by which the amplitude should be multiplied in order to
H:	normalize the volume of the audio file.  A value of zero means
H:	no amplification, 64 means the amplitude should be multiplied
H:	by a factor of two, 128 by three and so on.
H:
H:	This value is used by the amp filter.
H:
H: -v	Verbose mode. Explain what is being done.
H:
H: -p	Pathname match. Match a slash in the path only with a slash
H: 	in pattern and not by an asterisk (*) or a question mark
H: 	(?) metacharacter, nor by a bracket expression ([]) containing
H: 	a slash (see fnmatch(3)).
---
N: cpsi
P: AFS_READ | AFS_WRITE
D: Copy audio file selector info.
U: cpsi [-a] [-y] [-i] [-l] [-n] [-v] source pattern...
H: If no option, or only the -v option is given, all fields of
H: the audio file selector info are copied to all files
H: matching pattern.  Otherwise, only the given options are
H: taken into account.
H:
H: Options:
H:
H: -a	Copy attributes.
H:
H: -y	Copy the lyrics id.
H:
H: -i	Copy the image id.
H:
H: -l	Copy the lastplayed time.
H:
H: -n	Copy the numplayed count.
H:
H: -v	Verbose mode.
---
N: select
P: AFS_READ | AFS_WRITE
D: Activate a mood or a playlist.
U: select specifier/name
H: The specifier is either 'm' or 'p' to indicate whether a playlist or
H: a mood should be activated. Example:
H:
H:	select m/foo
H:
H: loads the mood named 'foo'.
---
T: add
N: add@member@
O: int com_add@member@(struct command_context *cc);
P: AFS_READ | AFS_WRITE
D: Read data from stdin and add it as a blob to the @member@ table.
U: add@member@ @member@_name
H: Read arbitrary binary data from stdin and send that data to
H: the audio file selector process which creates a new blob for
H: the data in the corresponding osl table.
H:
H: The names of the blobs of a table are unique. If an entry with the
H: given name already exists, its contents are replaced by the new data.
---
T: cat
N: cat@member@
O: int com_cat@member@(struct command_context *cc);
P: AFS_READ
D: Dump the contents of a blob of type @member@ to stdout.
U: cat@member@ @member@_name
H: This command may be used to retrieve the blob identified by
H: the given name from the corresponding osl table to which
H: they were previously added.
---
T: ls
N: ls@member@
O: int com_ls@member@(struct command_context *cc);
P: AFS_READ
D: List blobs of type @member@ matching a pattern.
U: ls@member@ [-i] [-l] [-r] [pattern]
H: Print a list of the names of all blobs in the corresponding
H: osl table which match the given pattern. If no pattern is
H: given, the full list is printed.
H:
H: Options:
H:
H: -i	Sort attributes by id. The default is to sort
H:	alphabetically by name.
H:
H: -l	Print a long listing containing both identifier and
H:	attribute name. The default is to print only the name.
H:
H: -r	Reverse sort order.
---
T: rm
N: rm@member@
O: int com_rm@member@(struct command_context *cc);
P: AFS_READ | AFS_WRITE
D: Remove blob(s) of type @member@ from the @member@ table.
U: rm@member@ pattern...
H: Remove all blobs from the corresponding table which match
H: any given pattern.
---
T: mv
N: mv@member@
O: int com_mv@member@(struct command_context *cc);
P: AFS_READ | AFS_WRITE
D: Rename a blob of type @member@.
U: mv@member@ old_@member@_name new_@member@_name
H: Rename the blob identified by the first name as the second name.
